@@ -115,7 +115,7 @@ MICROCOMPLETIONS:
 			\dtl delete to line  (used like ‹`5\dtl›. Like d5G in vim.)
 			\ca  cancel          (cancels any command you would have written and deletes the arguments. More civilized than spamming undo or backspace.)
 		WITHOUT ARGUMENTS:
-			\de  debugger                (toggles the existence of 'debug()' on the top of your prompt without moving your cursor)
+			\db  debugger                (toggles the existence of 'debug()' on the top of your prompt without moving your cursor)
 			\pu  pudb (other debugger)   (similar to \de, pudb is an alternative debugger, and this command also toggles a line on the top of the buffer. Whether you use this or \de is a matter of prefernece.)
 			\vi  vim                     (opens up the current buffer in vim. Edit your code, then save the temp file and close vim. Your cursor's position will be shared between vim and rp.)
 			\ed  editor                  (opens up the current buffer in some text editor; might be nano, might be vim. Depends on what you have. This command is an alternative to \vi)
