@@ -2,8 +2,17 @@ rp.rp_ptpython
 rp.prompt_toolkit
 rp.rp_ptpdb
 rp.libs.tetris
+rp.libs.jedi
 rp.libs.pyflann
 rp.libs.tetris.graphics.graphics
+rp.libs.jedi.plugins
+rp.libs.jedi.common
+rp.libs.jedi.evaluate
+rp.libs.jedi.api
+rp.libs.jedi.evaluate.gradual
+rp.libs.jedi.evaluate.context
+rp.libs.jedi.evaluate.compiled
+rp.libs.jedi.evaluate.compiled.subprocess
 rp.libs.pyflann.util
 rp.libs.pyflann.io
 rp.libs.pyflann.bindings
